@@ -239,14 +239,14 @@ def feedback(sys1, sys2=1, sign=-1):
     # its feedback member function.
     if isinstance(sys1, (int, float, complex, np.number)):
         if isinstance(sys2, tf.TransferFunction):
-            sys1 = tf._convertToTransferFunction(sys1)
+            sys1 = tf._convert_to_transfer_function(sys1)
         elif isinstance(sys2, ss.StateSpace):
             sys1 = ss._convertToStateSpace(sys1)
         elif isinstance(sys2, frd.FRD):
             sys1 = ss._convertToFRD(sys1)
         else: # sys2 is a scalar.
-            sys1 = tf._convertToTransferFunction(sys1)
-            sys2 = tf._convertToTransferFunction(sys2)
+            sys1 = tf._convert_to_transfer_function(sys1)
+            sys2 = tf._convert_to_transfer_function(sys2)
 
     return sys1.feedback(sys2, sign)
 
