@@ -11,9 +11,17 @@
 bode_dB = False                 # Bode plot magnitude units
 bode_deg = True                 # Bode Plot phase units
 bode_Hz = False                 # Bode plot frequency units
+bode_number_of_samples = None   # Bode plot number of samples
+bode_feature_periphery_decade = 1.0  # Bode plot feature periphery in decades
 
 # Set defaults to match MATLAB
 def use_matlab_defaults():
+    """
+    Use MATLAB compatible configuration settings
+
+    The following conventions are used:
+        * Bode plots plot gain in dB, phase in degrees, frequency in Hertz
+    """
     # Bode plot defaults
     global bode_dB; bode_dB = True
     global bode_deg; bode_deg = True
@@ -21,6 +29,13 @@ def use_matlab_defaults():
 
 # Set defaults to match FBS (Astrom and Murray)
 def use_fbs_defaults():
+    """
+    Use `Feedback Systems <http://fbsbook.org>`_ (FBS) compatible settings
+
+    The following conventions are used:
+        * Bode plots plot gain in powers of ten, phase in degrees,
+          frequency in Hertz
+    """
     # Bode plot defaults
     global bode_dB; bode_dB = False
     global bode_deg; bode_deg = True
