@@ -2,9 +2,6 @@
 Copyright (C) 2011 by Eike Welk.
 
 Test the control.matlab toolbox.
-
-NOTE: this script is not part of the standard python-control unit
-tests.  Needs to be integrated into unit test files.
 '''
 
 import unittest
@@ -19,6 +16,8 @@ from control.matlab import ss, step, impulse, initial, lsim, dcgain, \
                            ss2tf
 from control.statesp import _mimo2siso
 from control.timeresp import _check_convert_array
+from control.exception import slycot_check
+import warnings
 
 class TestControlMatlab(unittest.TestCase):
 
@@ -66,26 +65,26 @@ class TestControlMatlab(unittest.TestCase):
         D = zeros((2, 2))
         return A, B, C, D
 
-
     def test_dcgain(self):
         """Test function dcgain with different systems"""
-        #Test MIMO systems
-        A, B, C, D = self.make_MIMO_mats()
+        if slycot_check():
+            #Test MIMO systems
+            A, B, C, D = self.make_MIMO_mats()
 
-        gain1 = dcgain(ss(A, B, C, D))
-        gain2 = dcgain(A, B, C, D)
-        sys_tf = ss2tf(A, B, C, D)
-        gain3 = dcgain(sys_tf)
-        gain4 = dcgain(sys_tf.num, sys_tf.den)
-        #print("gain1:", gain1)
+            gain1 = dcgain(ss(A, B, C, D))
+            gain2 = dcgain(A, B, C, D)
+            sys_tf = ss2tf(A, B, C, D)
+            gain3 = dcgain(sys_tf)
+            gain4 = dcgain(sys_tf.num, sys_tf.den)
+            #print("gain1:", gain1)
 
-        assert_array_almost_equal(gain1,
-                                  array([[0.0269, 0.    ],
-                                         [0.    , 0.0269]]),
-                                  decimal=4)
-        assert_array_almost_equal(gain1, gain2)
-        assert_array_almost_equal(gain3, gain4)
-        assert_array_almost_equal(gain1, gain4)
+            assert_array_almost_equal(gain1,
+                                      array([[0.0269, 0.    ],
+                                             [0.    , 0.0269]]),
+                                      decimal=4)
+            assert_array_almost_equal(gain1, gain2)
+            assert_array_almost_equal(gain3, gain4)
+            assert_array_almost_equal(gain1, gain4)
 
         #Test SISO systems
         A, B, C, D = self.make_SISO_mats()
@@ -95,13 +94,13 @@ class TestControlMatlab(unittest.TestCase):
                                   array([[0.0269]]),
                                   decimal=4)
 
-
     def test_dcgain_2(self):
         """Test function dcgain with different systems"""
         #Create different forms of a SISO system
         A, B, C, D = self.make_SISO_mats()
-        Z, P, k = scipy.signal.ss2zpk(A, B, C, D)
         num, den = scipy.signal.ss2tf(A, B, C, D)
+        # numerator is only a constant here; pick it out to avoid numpy warning
+        Z, P, k = scipy.signal.tf2zpk(num[0][-1], den)
         sys_ss = ss(A, B, C, D)
 
         #Compute the gain with ``dcgain``
@@ -109,29 +108,22 @@ class TestControlMatlab(unittest.TestCase):
         gain_zpk = dcgain(Z, P, k)
         gain_numden = dcgain(np.squeeze(num), den)
         gain_sys_ss = dcgain(sys_ss)
-        print('gain_abcd:', gain_abcd, 'gain_zpk:', gain_zpk)
-        print('gain_numden:', gain_numden, 'gain_sys_ss:', gain_sys_ss)
+        # print('gain_abcd:', gain_abcd, 'gain_zpk:', gain_zpk)
+        # print('gain_numden:', gain_numden, 'gain_sys_ss:', gain_sys_ss)
 
         #Compute the gain with a long simulation
         t = linspace(0, 1000, 1000)
         y, _t = step(sys_ss, t)
         gain_sim = y[-1]
-        print('gain_sim:', gain_sim)
+        # print('gain_sim:', gain_sim)
 
         #All gain values must be approximately equal to the known gain
-        assert_array_almost_equal([gain_abcd[0,0],   gain_zpk[0,0],
-                                   gain_numden[0,0], gain_sys_ss[0,0], gain_sim],
+        assert_array_almost_equal([gain_abcd,   gain_zpk,
+                                   gain_numden, gain_sys_ss, gain_sim],
                                   [0.026948, 0.026948, 0.026948, 0.026948,
                                    0.026948],
                                   decimal=6)
 
-        #Test with MIMO system
-        A, B, C, D = self.make_MIMO_mats()
-        gain_mimo = dcgain(A, B, C, D)
-        print('gain_mimo: \n', gain_mimo)
-        assert_array_almost_equal(gain_mimo, [[0.026948, 0       ],
-                                              [0,        0.026948]], decimal=6)
-
     def test_step(self):
         """Test function ``step``."""
         figure(); plot_shape = (1, 3)
@@ -152,6 +144,9 @@ class TestControlMatlab(unittest.TestCase):
         t, y = step(sys, T, X0)
         plot(t, y)
 
+        # Test output of state vector
+        t, y, x = step(sys, return_x=True)
+
         #Test MIMO system
         A, B, C, D = self.make_MIMO_mats()
         sys = ss(A, B, C, D)
@@ -160,9 +155,6 @@ class TestControlMatlab(unittest.TestCase):
         t, y = step(sys)
         plot(t, y)
 
-        #show()
-
-    @unittest.skip("skipping test_impulse, need to update test")
     def test_impulse(self):
         A, B, C, D = self.make_SISO_mats()
         sys = ss(A, B, C, D)
@@ -182,8 +174,10 @@ class TestControlMatlab(unittest.TestCase):
         #Test system with direct feed-though, the function should print a warning.
         D = [[0.5]]
         sys_ft = ss(A, B, C, D)
-        t, y = impulse(sys_ft)
-        plot(t, y, label='Direct feedthrough D=[[0.5]]')
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            t, y = impulse(sys_ft)
+            plot(t, y, label='Direct feedthrough D=[[0.5]]')
 
         #Test MIMO system
         A, B, C, D = self.make_MIMO_mats()
@@ -367,7 +361,7 @@ class TestControlMatlab(unittest.TestCase):
 
     def assert_systems_behave_equal(self, sys1, sys2):
         '''
-        Test if the behavior of two Lti systems is equal. Raises ``AssertionError``
+        Test if the behavior of two LTI systems is equal. Raises ``AssertionError``
         if the systems are not equal.
 
         Works only for SISO systems.
@@ -378,12 +372,10 @@ class TestControlMatlab(unittest.TestCase):
         assert_array_almost_equal(dcgain(sys1), dcgain(sys2))
 
         #Results of ``step`` simulation must be the same too
-        t, y1 = step(sys1)
-        _t, y2 = step(sys2, t)
+        y1, t1 = step(sys1)
+        y2, t2 = step(sys2, t1)
         assert_array_almost_equal(y1, y2)
 
-    #! Old test; no longer functional?? (RMM, 3 Nov 2012)
-    @unittest.skip("skipping test_convert_MIMOto_SISO: need to update test")
     def test_convert_MIMO_to_SISO(self):
         '''Convert mimo to siso systems'''
         #Test with our usual systems --------------------------------------------
@@ -445,10 +437,10 @@ class TestControlMatlab(unittest.TestCase):
                                          warn_conversion=False)
         sys_siso_10 = _mimo2siso(sys_mimo, input=1, output=0,
                                          warn_conversion=False)
-        print("sys_siso_01 ---------------------------------------------")
-        print(sys_siso_01)
-        print("sys_siso_10 ---------------------------------------------")
-        print(sys_siso_10)
+        # print("sys_siso_01 ---------------------------------------------")
+        # print(sys_siso_01)
+        # print("sys_siso_10 ---------------------------------------------")
+        # print(sys_siso_10)
 
         #gain of converted system and equivalent SISO system must be the same
         self.assert_systems_behave_equal(sys_siso, sys_siso_01)
@@ -474,6 +466,4 @@ class TestControlMatlab(unittest.TestCase):
 
 if __name__ == '__main__':
     unittest.main()
-    show()
-    print("Test finished correctly!")
 # vi:ts=4:sw=4:expandtab
