@@ -1,78 +1,12 @@
 # timeresp.py - time-domain simulation routes
-"""
-Time domain simulation.
+"""Time domain simulation.
 
-This file contains a collection of functions that calculate
-time responses for linear systems.
+This file contains a collection of functions that calculate time
+responses for linear systems.
 
-.. _time-series-convention:
+See doc/conventions.rst#time-series-conventions_ for more information
+on how time series data are represented.
 
-Convention for Time Series
---------------------------
-
-This is a convention for function arguments and return values that
-represent time series: sequences of values that change over time. It
-is used throughout the library, for example in the functions
-:func:`forced_response`, :func:`step_response`, :func:`impulse_response`,
-and :func:`initial_response`.
-
-.. note::
-    This convention is different from the convention used in the library
-    :mod:`scipy.signal`. In Scipy's convention the meaning of rows and columns
-    is interchanged.  Thus, all 2D values must be transposed when they are
-    used with functions from :mod:`scipy.signal`.
-
-Types:
-
-    * **Arguments** can be **arrays**, **matrices**, or **nested lists**.
-    * **Return values** are **arrays** (not matrices).
-
-The time vector is either 1D, or 2D with shape (1, n)::
-
-      T = [[t1,     t2,     t3,     ..., tn    ]]
-
-Input, state, and output all follow the same convention. Columns are different
-points in time, rows are different components. When there is only one row, a
-1D object is accepted or returned, which adds convenience for SISO systems::
-
-      U = [[u1(t1), u1(t2), u1(t3), ..., u1(tn)]
-           [u2(t1), u2(t2), u2(t3), ..., u2(tn)]
-           ...
-           ...
-           [ui(t1), ui(t2), ui(t3), ..., ui(tn)]]
-
-      Same for X, Y
-
-So, U[:,2] is the system's input at the third point in time; and U[1] or U[1,:]
-is the sequence of values for the system's second input.
-
-The initial conditions are either 1D, or 2D with shape (j, 1)::
-
-     X0 = [[x1]
-           [x2]
-           ...
-           ...
-           [xj]]
-
-As all simulation functions return *arrays*, plotting is convenient::
-
-    t, y = step(sys)
-    plot(t, y)
-
-The output of a MIMO system can be plotted like this::
-
-    t, y, x = lsim(sys, u, t)
-    plot(t, y[0], label='y_0')
-    plot(t, y[1], label='y_1')
-
-The convention also works well with the state space form of linear systems. If
-``D`` is the feedthrough *matrix* of a linear system, and ``U`` is its input
-(*matrix* or *array*), then the feedthrough part of the system's response,
-can be computed like this::
-
-    ft = D * U
-
-----------------------------------------------------------------
 """
 
 """Copyright (c) 2011 by California Institute of Technology
@@ -120,10 +54,12 @@ import scipy as sp              # SciPy library (used all over)
 import numpy as np              # NumPy library
 from scipy.signal.ltisys import _default_response_times
 import warnings
-from .lti import Lti     # base class of StateSpace, TransferFunction
+from .lti import LTI     # base class of StateSpace, TransferFunction
 from .statesp import _convertToStateSpace, _mimo2simo, _mimo2siso
 from .lti import isdtime, isctime
 
+__all__ = ['forced_response', 'step_response', 'step_info', 'initial_response',
+           'impulse_response']
 
 # Helper function for checking array-like parameters
 def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
@@ -233,7 +169,8 @@ def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
 
 
 # Forced response of a linear system
-def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
+def forced_response(sys, T=None, U=0., X0=0., transpose=False,
+                    interpolate=False):
     """Simulate the output of a linear system.
 
     As a convenience for parameters `U`, `X0`:
@@ -241,16 +178,15 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
     The correct shape is inferred from arguments `sys` and `T`.
 
     For information on the **shape** of parameters `U`, `T`, `X0` and
-    return values `T`, `yout`, `xout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, `xout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
-    sys: Lti (StateSpace, or TransferFunction)
+    sys: LTI (StateSpace, or TransferFunction)
         LTI system to simulate
 
     T: array-like
-        Time steps at which the input is defined, numbers must be (strictly
-        monotonic) increasing.
+        Time steps at which the input is defined; values must be evenly spaced.
 
     U: array-like or number, optional
         Input array giving input at each time `T` (default = 0).
@@ -265,12 +201,12 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
+    interpolate:bool
+        If True and system is a discrete time system, the input will
+        be interpolated between the given time steps and the output
+        will be given at system sampling rate.  Otherwise, only return
+        the output at the times given in `T`.  No effect on continuous
+        time simulations (default = False).
 
     Returns
     -------
@@ -288,9 +224,11 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
     Examples
     --------
     >>> T, yout, xout = forced_response(sys, T, u, X0)
+
+    See :ref:`time-series-convention`.
     """
-    if not isinstance(sys, Lti):
-        raise TypeError('Parameter ``sys``: must be a ``Lti`` object. '
+    if not isinstance(sys, LTI):
+        raise TypeError('Parameter ``sys``: must be a ``LTI`` object. '
                         '(For example ``StateSpace`` or ``TransferFunction``)')
     sys = _convertToStateSpace(sys)
     A, B, C, D = np.asarray(sys.A), np.asarray(sys.B), np.asarray(sys.C), \
@@ -298,6 +236,7 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
 #    d_type = A.dtype
     n_states = A.shape[0]
     n_inputs = B.shape[1]
+    n_outputs = C.shape[0]
 
     # Set and/or check time vector in discrete time case
     if isdtime(sys, strict=True):
@@ -305,8 +244,8 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
             if U is None:
                 raise ValueError('Parameters ``T`` and ``U`` can\'t both be'
                                  'zero for discrete-time simulation')
-            # Set T to integers with same length as U
-            T = range(len(U))
+            # Set T to equally spaced samples with same length as U
+            T = np.array(range(len(U))) * (1 if sys.dt == True else sys.dt)
         else:
             # Make sure the input vector and time vector have same length
             # TODO: allow interpolation of the input vector
@@ -323,15 +262,19 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
     T = _check_convert_array(T, [('any',), (1, 'any')],
                              'Parameter ``T``: ', squeeze=True,
                              transpose=transpose)
-    if not all(T[1:] - T[:-1] > 0):
-        raise ValueError('Parameter ``T``: time values must be '
-                         '(strictly monotonic) increasing numbers.')
+    dt = T[1] - T[0]
+    if not np.allclose(T[1:] - T[:-1], dt):
+        raise ValueError('Parameter ``T``: time values must be equally spaced.')
     n_steps = len(T)            # number of simulation steps
 
     # create X0 if not given, test if X0 has correct shape
     X0 = _check_convert_array(X0, [(n_states,), (n_states, 1)],
                               'Parameter ``X0``: ', squeeze=True)
 
+    xout = np.zeros((n_states, n_steps))
+    xout[:, 0] = X0
+    yout = np.zeros((n_outputs, n_steps))
+
     # Separate out the discrete and continuous time cases
     if isctime(sys):
         # Solve the differential equation, copied from scipy.signal.ltisys.
@@ -339,12 +282,11 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
 
         # Faster algorithm if U is zero
         if U is None or (isinstance(U, (int, float)) and U == 0):
-            # Function that computes the time derivative of the linear system
-            def f_dot(x, _t):
-                return dot(A, x)
-
-            xout = sp.integrate.odeint(f_dot, X0, T, **keywords)
-            yout = dot(C, xout.T)
+            # Solve using matrix exponential
+            expAdt = sp.linalg.expm(A * dt)
+            for i in range(1, n_steps):
+                xout[:, i] = dot(expAdt, xout[:, i-1])
+            yout = dot(C, xout)
 
         # General algorithm that interpolates U in between output points
         else:
@@ -354,39 +296,81 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
             U = _check_convert_array(U, legal_shapes,
                                      'Parameter ``U``: ', squeeze=False,
                                      transpose=transpose)
-            # convert 1D array to D2 array with only one row
+            # convert 1D array to 2D array with only one row
             if len(U.shape) == 1:
                 U = U.reshape(1, -1)  # pylint: disable=E1103
 
-            # Create a callable that uses linear interpolation to
-            # calculate the input at any time.
-            compute_u = \
-                sp.interpolate.interp1d(T, U, kind='linear', copy=False,
-                                        axis=-1, bounds_error=False,
-                                        fill_value=0)
+            # Algorithm: to integrate from time 0 to time dt, with linear
+            # interpolation between inputs u(0) = u0 and u(dt) = u1, we solve
+            #   xdot = A x + B u,        x(0) = x0
+            #   udot = (u1 - u0) / dt,   u(0) = u0.
+            #
+            # Solution is
+            #   [ x(dt) ]       [ A*dt  B*dt  0 ] [  x0   ]
+            #   [ u(dt) ] = exp [  0     0    I ] [  u0   ]
+            #   [u1 - u0]       [  0     0    0 ] [u1 - u0]
 
-            # Function that computes the time derivative of the linear system
-            def f_dot(x, t):
-                return dot(A, x) + squeeze(dot(B, compute_u([t])))
+            M = np.bmat([[A * dt, B * dt, np.zeros((n_states, n_inputs))],
+                         [np.zeros((n_inputs, n_states + n_inputs)),
+                          np.identity(n_inputs)],
+                         [np.zeros((n_inputs, n_states + 2 * n_inputs))]])
+            expM = sp.linalg.expm(M)
+            Ad = expM[:n_states, :n_states]
+            Bd1 = expM[:n_states, n_states+n_inputs:]
+            Bd0 = expM[:n_states, n_states:n_states + n_inputs] - Bd1
 
-            xout = sp.integrate.odeint(f_dot, X0, T, **keywords)
-            yout = dot(C, xout.T) + dot(D, U)
+            for i in range(1, n_steps):
+                xout[:, i] = (dot(Ad, xout[:, i-1]) + dot(Bd0, U[:, i-1]) +
+                              dot(Bd1, U[:, i]))
+            yout = dot(C, xout) + dot(D, U)
 
+        tout = T
         yout = squeeze(yout)
-        xout = xout.T
+        xout = squeeze(xout)
 
     else:
+        # Discrete type system => use SciPy signal processing toolbox
+        if (sys.dt != True):
+            # Make sure that the time increment is a multiple of sampling time
+
+            # First make sure that time increment is bigger than sampling time
+            if dt < sys.dt:
+                raise ValueError("Time steps ``T`` must match sampling time")
+
+            # Now check to make sure it is a multiple (with check against
+            # sys.dt because floating point mod can have small errors
+            elif not (np.isclose(dt % sys.dt, 0) or
+                      np.isclose(dt % sys.dt, sys.dt)):
+                raise ValueError("Time steps ``T`` must be multiples of " \
+                                 "sampling time")
+        else:
+            sys.dt = dt         # For unspecified sampling time, use time incr
+
         # Discrete time simulation using signal processing toolbox
         dsys = (A, B, C, D, sys.dt)
-        tout, yout, xout = sp.signal.dlsim(dsys, U, T, X0)
+
+        # Use signal processing toolbox for the discrete time simulation
+        # Transpose the input to match toolbox convention 
+        tout, yout, xout = sp.signal.dlsim(dsys, np.transpose(U), T, X0)
+
+        if not interpolate:
+            # If dt is different from sys.dt, resample the output
+            inc = int(round(dt / sys.dt))
+            tout = T            # Return exact list of time steps
+            yout = yout[::inc,:]
+            xout = xout[::inc,:]
+
+        # Transpose the output and state vectors to match local convention
+        xout = sp.transpose(xout)
+        yout = sp.transpose(yout)
 
     # See if we need to transpose the data back into MATLAB form
     if (transpose):
-        T = np.transpose(T)
+        tout = np.transpose(tout)
         yout = np.transpose(yout)
         xout = np.transpose(xout)
 
-    return T, yout, xout
+    return tout, yout, xout
 
 def _get_ss_simo(sys, input=None, output=None):
     """Return a SISO or SIMO state-space version of sys
@@ -407,7 +391,7 @@ def _get_ss_simo(sys, input=None, output=None):
         return _mimo2siso(sys_ss, input, output, warn_conversion=warn)
 
 def step_response(sys, T=None, X0=0., input=None, output=None,
-                  transpose=False, **keywords):
+                  transpose=False, return_x=False):
     # pylint: disable=W0622
     """Step response of a linear system
 
@@ -417,7 +401,7 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     inputs are set to 0, all other outputs are ignored.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -443,13 +427,8 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -459,6 +438,9 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     yout: array
         Response of the system
 
+    xout: array
+        Individual response of each x variable
+
     See Also
     --------
     forced_response, initial_response, impulse_response
@@ -478,14 +460,109 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
 
     U = np.ones_like(T)
 
-    T, yout, _xout = forced_response(sys, T, U, X0,
-                                     transpose=transpose, **keywords)
+    T, yout, xout = forced_response(sys, T, U, X0,
+                                     transpose=transpose)
+
+    if return_x:
+        return T, yout, xout
 
     return T, yout
 
+def step_info(sys, T=None, SettlingTimeThreshold=0.02, RiseTimeLimits=(0.1,0.9)):
+    '''
+    Step response characteristics (Rise time, Settling Time, Peak and others).
+
+    Parameters
+    ----------
+    sys: StateSpace, or TransferFunction
+        LTI system to simulate
+
+    T: array-like object, optional
+        Time vector (argument is autocomputed if not given)
+
+    SettlingTimeThreshold: float value, optional
+        Defines the error to compute settling time (default = 0.02)
+
+    RiseTimeLimits: tuple (lower_threshold, upper_theshold)
+        Defines the lower and upper threshold for RiseTime computation
+
+    Returns
+    -------
+    S: a dictionary containing:
+        RiseTime: Time from 10% to 90% of the steady-state value.
+        SettlingTime: Time to enter inside a default error of 2%
+        SettlingMin: Minimum value after RiseTime
+        SettlingMax: Maximum value after RiseTime
+        Overshoot: Percentage of the Peak relative to steady value
+        Undershoot: Percentage of undershoot
+        Peak: Absolute peak value
+        PeakTime: time of the Peak
+        SteadyStateValue: Steady-state value
+
+
+    See Also
+    --------
+    step, lsim, initial, impulse
+
+    Examples
+    --------
+    >>> info = step_info(sys, T)
+    '''
+    sys = _get_ss_simo(sys)
+    if T is None:
+        if isctime(sys):
+            T = _default_response_times(sys.A, 1000)
+        else:
+            # For discrete time, use integers
+            tvec = _default_response_times(sys.A, 1000)
+            T = range(int(np.ceil(max(tvec))))
+
+    T, yout = step_response(sys, T)
+
+    # Steady state value
+    InfValue = yout[-1]
+
+    # RiseTime
+    tr_lower_index = (np.where(yout >= RiseTimeLimits[0] * InfValue)[0])[0]
+    tr_upper_index = (np.where(yout >= RiseTimeLimits[1] * InfValue)[0])[0]
+    RiseTime = T[tr_upper_index] - T[tr_lower_index]
+
+    # SettlingTime
+    sup_margin = (1. + SettlingTimeThreshold) * InfValue
+    inf_margin = (1. - SettlingTimeThreshold) * InfValue
+    # find Steady State looking for the first point out of specified limits
+    for i in reversed(range(T.size)):
+        if((yout[i] <= inf_margin) | (yout[i] >= sup_margin)):
+            SettlingTime = T[i + 1]
+            break
+
+    # Peak
+    PeakIndex = np.abs(yout).argmax()
+    PeakValue = yout[PeakIndex]
+    PeakTime = T[PeakIndex]
+    SettlingMax = (yout).max()
+    SettlingMin = (yout[tr_upper_index:]).min()
+    # I'm really not very confident about UnderShoot:
+    UnderShoot = yout.min()
+    OverShoot = 100. * (yout.max() - InfValue) / (InfValue - yout[0])
+
+    # Return as a dictionary
+    S = {
+        'RiseTime': RiseTime,
+        'SettlingTime': SettlingTime,
+        'SettlingMin': SettlingMin,
+        'SettlingMax': SettlingMax,
+        'Overshoot': OverShoot,
+        'Undershoot': UnderShoot,
+        'Peak': PeakValue,
+        'PeakTime': PeakTime,
+        'SteadyStateValue': InfValue
+    }
+
+    return S
 
 def initial_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, **keywords):
+                     transpose=False, return_x=False):
     # pylint: disable=W0622
     """Initial condition response of a linear system
 
@@ -494,7 +571,7 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
     outputs are given.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -521,14 +598,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
-
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -536,6 +607,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
         Time values of the output
     yout: array
         Response of the system
+    xout: array
+        Individual response of each x variable
 
     See Also
     --------
@@ -550,16 +623,24 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
     # Create time and input vectors; checking is done in forced_response(...)
     # The initial vector X0 is created in forced_response(...) if necessary
     if T is None:
-        T = _default_response_times(sys.A, 100)
+        if isctime(sys):
+            T = _default_response_times(sys.A, 1000)
+        else:
+            # For discrete time, use integers
+            tvec = _default_response_times(sys.A, 1000)
+            T = range(int(np.ceil(max(tvec))))
     U = np.zeros_like(T)
 
-    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose,
-                                     **keywords)
+    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose)
+
+    if return_x:
+        return T, yout, _xout
+
     return T, yout
 
 
 def impulse_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, **keywords):
+                     transpose=False, return_x=False):
     # pylint: disable=W0622
     """Impulse response of a linear system
 
@@ -569,7 +650,7 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     inputs are set to 0, all other outputs are ignored.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -595,14 +676,8 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
-
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -610,10 +685,12 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
         Time values of the output
     yout: array
         Response of the system
+    xout: array
+        Individual response of each x variable
 
     See Also
     --------
-    ForcedReponse, initial_response, step_response
+    forced_response, initial_response, step_response
 
     Examples
     --------
@@ -633,19 +710,33 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     X0 = _check_convert_array(X0, [(n_states,), (n_states, 1)],
                               'Parameter ``X0``: \n', squeeze=True)
 
+    # Compute T and U, no checks necessary, they will be checked in lsim
+    if T is None:
+        if isctime(sys):
+            T = _default_response_times(sys.A, 100)
+        else:
+            # For discrete time, use integers
+            tvec = _default_response_times(sys.A, 100)
+            T = range(int(np.ceil(max(tvec))))
+
+    U = np.zeros_like(T)
+
     # Compute new X0 that contains the impulse
     # We can't put the impulse into U because there is no numerical
     # representation for it (infinitesimally short, infinitely high).
     # See also: http://www.mathworks.com/support/tech-notes/1900/1901.html
-    B = np.asarray(sys.B).squeeze()
-    new_X0 = B + X0
-
-    # Compute T and U, no checks necessary, they will be checked in lsim
-    if T is None:
-        T = _default_response_times(sys.A, 100)
-    U = np.zeros_like(T)
+    if isctime(sys):
+        B = np.asarray(sys.B).squeeze()
+        new_X0 = B + X0
+    else:
+        new_X0 = X0
+        U[0] = 1.
 
     T, yout, _xout = forced_response(
         sys, T, U, new_X0,
-        transpose=transpose, **keywords)
+        transpose=transpose)
+
+    if return_x:
+        return T, yout, _xout
+
     return T, yout
