@@ -53,24 +53,24 @@ $Id$
 
 # External function declarations
 import numpy as np
-from numpy import angle, any, array, empty, finfo, insert, ndarray, ones, \
-    polyadd, polymul, polyval, roots, sort, sqrt, zeros, squeeze, exp, pi, \
-    where, delete, real, poly, poly1d, nonzero
+from numpy import angle, array, empty, finfo, ndarray, ones, \
+    polyadd, polymul, polyval, roots, sqrt, zeros, squeeze, exp, pi, \
+    where, delete, real, poly, nonzero
 import scipy as sp
 from numpy.polynomial.polynomial import polyfromroots
 from scipy.signal import lti, tf2zpk, zpk2tf, cont2discrete
 from copy import deepcopy
-import warnings
 from warnings import warn
 from itertools import chain
 from .lti import LTI, timebaseEqual, timebase, isdtime
 
 __all__ = ['TransferFunction', 'tf', 'ss2tf', 'tfdata']
 
+
 class TransferFunction(LTI):
 
     """TransferFunction(num, den[, dt])
-    
+
     A class for representing transfer functions
 
     The TransferFunction class is used to represent systems in transfer function
@@ -129,29 +129,29 @@ class TransferFunction(LTI):
             raise ValueError("Needs 1, 2 or 3 arguments; received %i."
                              % len(args))
 
-        num = _cleanPart(num)
-        den = _cleanPart(den)
+        num = _clean_part(num)
+        den = _clean_part(den)
 
         inputs = len(num[0])
         outputs = len(num)
 
         # Make sure numerator and denominator matrices have consistent sizes
         if inputs != len(den[0]):
-            raise ValueError("The numerator has %i input(s), but the \
-denominator has %i\ninput(s)." % (inputs, len(den[0])))
+            raise ValueError("The numerator has %i input(s), but the denominator has "
+                             "%i\ninput(s)." % (inputs, len(den[0])))
         if outputs != len(den):
-            raise ValueError("The numerator has %i output(s), but the \
-denominator has %i\noutput(s)." % (outputs, len(den)))
+            raise ValueError("The numerator has %i output(s), but the denominator has "
+                             "%i\noutput(s)." % (outputs, len(den)))
 
-        # Additional checks/updates on structure of the transfer function 
+        # Additional checks/updates on structure of the transfer function
         for i in range(outputs):
             # Make sure that each row has the same number of columns
             if len(num[i]) != inputs:
-                raise ValueError("Row 0 of the numerator matrix has %i \
-elements, but row %i\nhas %i." % (inputs, i, len(num[i])))
+                raise ValueError("Row 0 of the numerator matrix has %i elements, but row "
+                                 "%i\nhas %i." % (inputs, i, len(num[i])))
             if len(den[i]) != inputs:
-                raise ValueError("Row 0 of the denominator matrix has %i \
-elements, but row %i\nhas %i." % (inputs, i, len(den[i])))
+                raise ValueError("Row 0 of the denominator matrix has %i elements, but row "
+                                 "%i\nhas %i." % (inputs, i, len(den[i])))
 
             # Check for zeros in numerator or denominator
             # TODO: Right now these checks are only done during construction.
@@ -165,8 +165,8 @@ elements, but row %i\nhas %i." % (inputs, i, len(den[i])))
                         zeroden = False
                         break
                 if zeroden:
-                    raise ValueError("Input %i, output %i has a zero \
-denominator." % (j + 1, i + 1))
+                    raise ValueError("Input %i, output %i has a zero denominator."
+                                     % (j + 1, i + 1))
 
                 # If we have zero numerators, set the denominator to 1.
                 zeronum = True
@@ -214,7 +214,7 @@ denominator." % (j + 1, i + 1))
                     # Find the first nontrivial coefficient.
                     nonzero = None
                     for k in range(data[p][i][j].size):
-                        if (data[p][i][j][k]):
+                        if data[p][i][j][k]:
                             nonzero = k
                             break
 
@@ -230,7 +230,7 @@ denominator." % (j + 1, i + 1))
         """String representation of the transfer function."""
 
         mimo = self.inputs > 1 or self.outputs > 1
-        if (var is None):
+        if var is None:
             #! TODO: replace with standard calls to lti functions
             var = 's' if self.dt is None or self.dt == 0 else 'z'
         outstr = ""
@@ -241,8 +241,8 @@ denominator." % (j + 1, i + 1))
                     outstr += "\nInput %i to output %i:" % (i + 1, j + 1)
 
                 # Convert the numerator and denominator polynomials to strings.
-                numstr = _tfpolyToString(self.num[j][i], var=var)
-                denstr = _tfpolyToString(self.den[j][i], var=var)
+                numstr = _tf_polynomial_to_string(self.num[j][i], var=var)
+                denstr = _tf_polynomial_to_string(self.den[j][i], var=var)
 
                 # Figure out the length of the separating line
                 dashcount = max(len(numstr), len(denstr))
@@ -250,17 +250,17 @@ denominator." % (j + 1, i + 1))
 
                 # Center the numerator or denominator
                 if len(numstr) < dashcount:
-                    numstr = (' ' * int(round((dashcount - len(numstr))/2)) +
+                    numstr = (' ' * int(round((dashcount - len(numstr)) / 2)) +
                               numstr)
                 if len(denstr) < dashcount:
-                    denstr = (' ' * int(round((dashcount - len(denstr))/2)) +
+                    denstr = (' ' * int(round((dashcount - len(denstr)) / 2)) +
                               denstr)
 
                 outstr += "\n" + numstr + "\n" + dashes + "\n" + denstr + "\n"
 
         # See if this is a discrete time system with specific sampling time
-        if (not (self.dt is None) and type(self.dt) != bool and self.dt > 0):
-            #! TODO: replace with standard calls to lti functions
+        if not (self.dt is None) and type(self.dt) != bool and self.dt > 0:
+            # TODO: replace with standard calls to lti functions
             outstr += "\ndt = " + self.dt.__str__() + "\n"
 
         return outstr
@@ -268,6 +268,45 @@ denominator." % (j + 1, i + 1))
     # represent as string, makes display work for IPython
     __repr__ = __str__
 
+    def _repr_latex_(self, var=None):
+        """LaTeX representation of the transfer function, for Jupyter notebook"""
+
+        mimo = self.inputs > 1 or self.outputs > 1
+
+        if var is None:
+            # ! TODO: replace with standard calls to lti functions
+            var = 's' if self.dt is None or self.dt == 0 else 'z'
+
+        out = ['$$']
+
+        if mimo:
+            out.append(r"\begin{bmatrix}")
+
+        for i in range(self.outputs):
+            for j in range(self.inputs):
+                # Convert the numerator and denominator polynomials to strings.
+                numstr = _tf_polynomial_to_string(self.num[i][j], var=var)
+                denstr = _tf_polynomial_to_string(self.den[i][j], var=var)
+
+                out += [r"\frac{", numstr, "}{", denstr, "}"]
+
+                if mimo and j < self.outputs - 1:
+                    out.append("&")
+
+            if mimo:
+                out.append(r"\\")
+
+        if mimo:
+            out.append(r" \end{bmatrix}")
+
+        # See if this is a discrete time system with specific sampling time
+        if not (self.dt is None) and type(self.dt) != bool and self.dt > 0:
+            out += ["\quad dt = ", str(self.dt)]
+
+        out.append("$$")
+
+        return ''.join(out)
+
     def __neg__(self):
         """Negate a transfer function."""
 
@@ -283,25 +322,24 @@ denominator." % (j + 1, i + 1))
         from .statesp import StateSpace
 
         # Convert the second argument to a transfer function.
-        if (isinstance(other, StateSpace)):
-            other = _convertToTransferFunction(other)
+        if isinstance(other, StateSpace):
+            other = _convert_to_transfer_function(other)
         elif not isinstance(other, TransferFunction):
-            other = _convertToTransferFunction(other, inputs=self.inputs,
-                                               outputs=self.outputs)
+            other = _convert_to_transfer_function(other, inputs=self.inputs,
+                                                  outputs=self.outputs)
 
         # Check that the input-output sizes are consistent.
         if self.inputs != other.inputs:
-            raise ValueError("The first summand has %i input(s), but the \
-second has %i." % (self.inputs, other.inputs))
+            raise ValueError("The first summand has %i input(s), but the second has %i."
+                             % (self.inputs, other.inputs))
         if self.outputs != other.outputs:
-            raise ValueError("The first summand has %i output(s), but the \
-second has %i." % (self.outputs, other.outputs))
+            raise ValueError("The first summand has %i output(s), but the second has %i."
+                             % (self.outputs, other.outputs))
 
         # Figure out the sampling time to use
-        if (self.dt is None and other.dt is not None):
+        if self.dt is None and other.dt is not None:
             dt = other.dt       # use dt from second argument
-        elif (other.dt is None and self.dt is not None) or \
-                (timebaseEqual(self, other)):
+        elif (other.dt is None and self.dt is not None) or (timebaseEqual(self, other)):
             dt = self.dt        # use dt from first argument
         else:
             raise ValueError("Systems have different sampling times")
@@ -312,9 +350,9 @@ second has %i." % (self.outputs, other.outputs))
 
         for i in range(self.outputs):
             for j in range(self.inputs):
-                num[i][j], den[i][j] = _addSISO(self.num[i][j], self.den[i][j],
-                                                other.num[i][j],
-                                                other.den[i][j])
+                num[i][j], den[i][j] = _add_siso(self.num[i][j], self.den[i][j],
+                                                 other.num[i][j],
+                                                 other.den[i][j])
 
         return TransferFunction(num, den, dt)
 
@@ -334,24 +372,23 @@ second has %i." % (self.outputs, other.outputs))
         """Multiply two LTI objects (serial connection)."""
         # Convert the second argument to a transfer function.
         if isinstance(other, (int, float, complex, np.number)):
-            other = _convertToTransferFunction(other, inputs=self.inputs,
-                                               outputs=self.inputs)
+            other = _convert_to_transfer_function(other, inputs=self.inputs,
+                                                  outputs=self.inputs)
         else:
-            other = _convertToTransferFunction(other)
+            other = _convert_to_transfer_function(other)
 
         # Check that the input-output sizes are consistent.
         if self.inputs != other.outputs:
-            raise ValueError("C = A * B: A has %i column(s) (input(s)), but B \
-has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
+            raise ValueError("C = A * B: A has %i column(s) (input(s)), but B has %i "
+                             "row(s)\n(output(s))." % (self.inputs, other.outputs))
 
         inputs = other.inputs
         outputs = self.outputs
 
         # Figure out the sampling time to use
-        if (self.dt is None and other.dt is not None):
+        if self.dt is None and other.dt is not None:
             dt = other.dt       # use dt from second argument
-        elif (other.dt is None and self.dt is not None) or \
-                (self.dt == other.dt):
+        elif (other.dt is None and self.dt is not None) or (self.dt == other.dt):
             dt = self.dt        # use dt from first argument
         else:
             raise ValueError("Systems have different sampling times")
@@ -360,18 +397,18 @@ has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
         num = [[[0] for j in range(inputs)] for i in range(outputs)]
         den = [[[1] for j in range(inputs)] for i in range(outputs)]
 
-        # Temporary storage for the summands needed to
-        # find the (i, j)th element of the product.
+        # Temporary storage for the summands needed to find the (i, j)th element of the product.
         num_summand = [[] for k in range(self.inputs)]
         den_summand = [[] for k in range(self.inputs)]
 
-        for i in range(outputs):  # Iterate through rows of product.
-            for j in range(inputs):  # Iterate through columns of product.
-                for k in range(self.inputs):  # Multiply & add.
-                    num_summand[k] = polymul(self.num[i][k], other.num[k][j])
-                    den_summand[k] = polymul(self.den[i][k], other.den[k][j])
-                    num[i][j], den[i][j] = _addSISO(
-                        num[i][j], den[i][j],
+        # Multiply & add.
+        for row in range(outputs):
+            for col in range(inputs):
+                for k in range(self.inputs):
+                    num_summand[k] = polymul(self.num[row][k], other.num[k][col])
+                    den_summand[k] = polymul(self.den[row][k], other.den[k][col])
+                    num[row][col], den[row][col] = _add_siso(
+                        num[row][col], den[row][col],
                         num_summand[k], den_summand[k])
 
         return TransferFunction(num, den, dt)
@@ -381,21 +418,21 @@ has %i row(s)\n(output(s))." % (self.inputs, other.outputs))
 
         # Convert the second argument to a transfer function.
         if isinstance(other, (int, float, complex, np.number)):
-            other = _convertToTransferFunction(other, inputs=self.inputs,
-                                               outputs=self.inputs)
+            other = _convert_to_transfer_function(other, inputs=self.inputs,
+                                                  outputs=self.inputs)
         else:
-            other = _convertToTransferFunction(other)
+            other = _convert_to_transfer_function(other)
 
         # Check that the input-output sizes are consistent.
         if other.inputs != self.outputs:
-            raise ValueError("C = A * B: A has %i column(s) (input(s)), but B \
-has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
+            raise ValueError("C = A * B: A has %i column(s) (input(s)), but B has %i "
+                             "row(s)\n(output(s))." % (other.inputs, self.outputs))
 
         inputs = self.inputs
         outputs = other.outputs
 
         # Figure out the sampling time to use
-        if (self.dt is None and other.dt is not None):
+        if self.dt is None and other.dt is not None:
             dt = other.dt       # use dt from second argument
         elif (other.dt is None and self.dt is not None) \
                 or (self.dt == other.dt):
@@ -418,7 +455,7 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
                 for k in range(other.inputs):  # Multiply & add.
                     num_summand[k] = polymul(other.num[i][k], self.num[k][j])
                     den_summand[k] = polymul(other.den[i][k], self.den[k][j])
-                    num[i][j], den[i][j] = _addSISO(
+                    num[i][j], den[i][j] = _add_siso(
                         num[i][j], den[i][j],
                         num_summand[k], den_summand[k])
 
@@ -429,11 +466,11 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
         """Divide two LTI objects."""
 
         if isinstance(other, (int, float, complex, np.number)):
-            other = _convertToTransferFunction(
+            other = _convert_to_transfer_function(
                 other, inputs=self.inputs,
                 outputs=self.inputs)
         else:
-            other = _convertToTransferFunction(other)
+            other = _convert_to_transfer_function(other)
 
         if (self.inputs > 1 or self.outputs > 1 or
                 other.inputs > 1 or other.outputs > 1):
@@ -442,10 +479,9 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
                 implemented only for SISO systems.")
 
         # Figure out the sampling time to use
-        if (self.dt is None and other.dt is not None):
+        if self.dt is None and other.dt is not None:
             dt = other.dt       # use dt from second argument
-        elif (other.dt is None and self.dt is not None)\
-                or (self.dt == other.dt):
+        elif (other.dt is None and self.dt is not None) or (self.dt == other.dt):
             dt = self.dt        # use dt from first argument
         else:
             raise ValueError("Systems have different sampling times")
@@ -463,17 +499,16 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
     def __rtruediv__(self, other):
         """Right divide two LTI objects."""
         if isinstance(other, (int, float, complex, np.number)):
-            other = _convertToTransferFunction(
+            other = _convert_to_transfer_function(
                 other, inputs=self.inputs,
                 outputs=self.inputs)
         else:
-            other = _convertToTransferFunction(other)
+            other = _convert_to_transfer_function(other)
 
         if (self.inputs > 1 or self.outputs > 1 or
                 other.inputs > 1 or other.outputs > 1):
             raise NotImplementedError(
-                "TransferFunction.__rtruediv__ is currently \
-                implemented only for SISO systems.")
+                "TransferFunction.__rtruediv__ is currently implemented only for SISO systems.")
 
         return other / self
 
@@ -487,9 +522,9 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
         if other == 0:
             return TransferFunction([1], [1])  # unity
         if other > 0:
-            return self * (self**(other-1))
+            return self * (self**(other - 1))
         if other < 0:
-            return (TransferFunction([1], [1]) / self) * (self**(other+1))
+            return (TransferFunction([1], [1]) / self) * (self**(other + 1))
 
     def __getitem__(self, key):
         key1, key2 = key
@@ -586,7 +621,6 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
         transfer function matrix evaluated at s = i * omega, where omega is a
         list of angular frequencies, and is a sorted
         version of the input omega.
-
         """
 
         # Preallocate outputs.
@@ -599,7 +633,7 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
         if isdtime(self, strict=True):
             dt = timebase(self)
             slist = np.array([exp(1.j * w * dt) for w in omega])
-            if (max(omega) * dt > pi):
+            if max(omega) * dt > pi:
                 warn("freqresp: frequency evaluation above Nyquist frequency")
         else:
             slist = np.array([1j * w for w in omega])
@@ -618,34 +652,33 @@ has %i row(s)\n(output(s))." % (other.inputs, self.outputs))
         """Compute the poles of a transfer function."""
         num, den, denorder = self._common_den()
         rts = []
-        for d, o in zip(den,denorder):
-            rts.extend(roots(d[:o+1]))
+        for d, o in zip(den, denorder):
+            rts.extend(roots(d[:o + 1]))
         return np.array(rts)
 
     def zero(self):
         """Compute the zeros of a transfer function."""
         if self.inputs > 1 or self.outputs > 1:
-            raise NotImplementedError("TransferFunction.zero is currently \
-only implemented for SISO systems.")
+            raise NotImplementedError("TransferFunction.zero is currently only implemented "
+                                      "for SISO systems.")
         else:
-            #for now, just give zeros of a SISO tf
+            # for now, just give zeros of a SISO tf
             return roots(self.num[0][0])
 
     def feedback(self, other=1, sign=-1):
         """Feedback interconnection between two LTI objects."""
-        other = _convertToTransferFunction(other)
+        other = _convert_to_transfer_function(other)
 
         if (self.inputs > 1 or self.outputs > 1 or
                 other.inputs > 1 or other.outputs > 1):
             # TODO: MIMO feedback
-            raise NotImplementedError("TransferFunction.feedback is currently \
-only implemented for SISO functions.")
+            raise NotImplementedError("TransferFunction.feedback is currently only implemented "
+                                      "for SISO functions.")
 
         # Figure out the sampling time to use
-        if (self.dt is None and other.dt is not None):
+        if self.dt is None and other.dt is not None:
             dt = other.dt       # use dt from second argument
-        elif (other.dt is None and self.dt is not None) \
-                or (self.dt == other.dt):
+        elif (other.dt is None and self.dt is not None) or (self.dt == other.dt):
             dt = self.dt        # use dt from first argument
         else:
             raise ValueError("Systems have different sampling times")
@@ -719,7 +752,7 @@ only implemented for SISO functions.")
         """
 
         # TODO: implement for discrete time systems
-        if (self.dt != 0 and self.dt is not None):
+        if self.dt != 0 and self.dt is not None:
             raise NotImplementedError("Function not \
                     implemented in discrete time")
 
@@ -732,7 +765,6 @@ only implemented for SISO functions.")
 
         return out
 
-
     def _common_den(self, imag_tol=None):
         """
         Compute MIMO common denominators; return them and adjusted numerators.
@@ -740,8 +772,8 @@ only implemented for SISO functions.")
         This function computes the denominators per input containing all
         the poles of sys.den, and reports it as the array den.  The
         output numerator array num is modified to use the common
-        denominator for this input/column; the coefficient arrays are also 
-        padded with zeros to be the same size for all num/den.  
+        denominator for this input/column; the coefficient arrays are also
+        padded with zeros to be the same size for all num/den.
         num is an sys.outputs by sys.inputs
         by len(d) array.
 
@@ -760,12 +792,12 @@ only implemented for SISO functions.")
             order; highest coefficient starts on the left.
 
         den: array
-            Multi-dimensional array of coefficients for common denominator 
+            Multi-dimensional array of coefficients for common denominator
             polynomial, one row per input. The array is prepared for use in
             slycot td04ad, the first element is the highest-order polynomial
             coefficiend of s, matching the order in denorder, if denorder <
             number of columns in den, the den is padded with zeros
-            
+
         denorder: array of int, orders of den, one per input
 
 
@@ -785,12 +817,12 @@ only implemented for SISO functions.")
 
         # A list to keep track of cumulative poles found as we scan
         # self.den[..][..]
-        poles = [ [] for j in range(self.inputs) ]
+        poles = [[] for j in range(self.inputs)]
 
         # RvP, new implementation 180526, issue #194
 
         # pre-calculate the poles for all num, den
-        # has zeros, poles, gain, list for pole indices not in den, 
+        # has zeros, poles, gain, list for pole indices not in den,
         # number of poles known at the time analyzed
 
         # do not calculate minreal. Rory's hint .minreal()
@@ -799,12 +831,12 @@ only implemented for SISO functions.")
             poleset.append([])
             for j in range(self.inputs):
                 if abs(self.num[i][j]).max() <= eps:
-                    poleset[-1].append( [array([], dtype=float),
-                               roots(self.den[i][j]), 0.0, [], 0 ])
+                    poleset[-1].append([array([], dtype=float),
+                                        roots(self.den[i][j]), 0.0, [], 0])
                 else:
                     z, p, k = tf2zpk(self.num[i][j], self.den[i][j])
-                    poleset[-1].append([ z, p, k, [], 0])
-        
+                    poleset[-1].append([z, p, k, [], 0])
+
         # collect all individual poles
         epsnm = eps * self.inputs * self.outputs
         for j in range(self.inputs):
@@ -827,51 +859,50 @@ only implemented for SISO functions.")
 
         # figure out maximum number of poles, for sizing the den
         npmax = max([len(p) for p in poles])
-        den = zeros((self.inputs, npmax+1), dtype=float)
-        num = zeros((max(1,self.outputs,self.inputs), 
-                     max(1,self.outputs,self.inputs), npmax+1), dtype=float)
+        den = zeros((self.inputs, npmax + 1), dtype=float)
+        num = zeros((max(1, self.outputs, self.inputs),
+                     max(1, self.outputs, self.inputs), npmax + 1), dtype=float)
         denorder = zeros((self.inputs,), dtype=int)
-        
+
         for j in range(self.inputs):
             if not len(poles[j]):
                 # no poles matching this input; only one or more gains
-                den[j,0] = 1.0
+                den[j, 0] = 1.0
                 for i in range(self.outputs):
-                    num[i,j,0] = poleset[i][j][2]
+                    num[i, j, 0] = poleset[i][j][2]
             else:
                 # create the denominator matching this input
                 # polyfromroots gives coeffs in opposite order from what we use
                 # coefficients should be padded on right, ending at np
                 np = len(poles[j])
-                den[j,np::-1] = polyfromroots(poles[j]).real
+                den[j, np::-1] = polyfromroots(poles[j]).real
                 denorder[j] = np
 
                 # now create the numerator, also padded on the right
                 for i in range(self.outputs):
                     # start with the current set of zeros for this output
                     nwzeros = list(poleset[i][j][0])
-                    # add all poles not found in the original denominator, 
+                    # add all poles not found in the original denominator,
                     # and the ones later added from other denominators
                     for ip in chain(poleset[i][j][3],
-                                    range(poleset[i][j][4],np)):
+                                    range(poleset[i][j][4], np)):
                         nwzeros.append(poles[j][ip])
 
-                    numpoly = poleset[i][j][2] * polyfromroots(nwzeros).real 
+                    numpoly = poleset[i][j][2] * polyfromroots(nwzeros).real
                     # print(numpoly, den[j])
                     # polyfromroots gives coeffs in opposite order => invert
                     # numerator polynomial should be padded on left and right
                     #   ending at np to line up with what td04ad expects...
-                    num[i, j, np+1-len(numpoly):np+1] = numpoly[::-1]
+                    num[i, j, np + 1 - len(numpoly):np + 1] = numpoly[::-1]
                     # print(num[i, j])
 
         if (abs(den.imag) > epsnm).any():
             print("Warning: The denominator has a nontrivial imaginary part: %f"
-                      % abs(den.imag).max())
+                  % abs(den.imag).max())
         den = den.real
 
         return num, den, denorder
 
-        
     def sample(self, Ts, method='zoh', alpha=None):
         """Convert a continuous-time system to discrete time
 
@@ -917,10 +948,10 @@ only implemented for SISO functions.")
         if not self.issiso():
             raise NotImplementedError("MIMO implementation not available")
         if method == "matched":
-            return _c2dmatched(self, Ts)
+            return _c2d_matched(self, Ts)
         sys = (self.num[0][0], self.den[0][0])
         numd, dend, dt = cont2discrete(sys, Ts, method, alpha)
-        return TransferFunction(numd[0,:], dend, dt)
+        return TransferFunction(numd[0, :], dend, dt)
 
     def dcgain(self):
         """Return the zero-frequency (or DC) gain
@@ -941,7 +972,7 @@ only implemented for SISO functions.")
     def _dcgain_cont(self):
         """_dcgain_cont() -> DC gain as matrix or scalar
 
-        Special cased evaluation at 0 for continuous-time systems"""
+        Special cased evaluation at 0 for continuous-time systems."""
         gain = np.empty((self.outputs, self.inputs), dtype=float)
         for i in range(self.outputs):
             for j in range(self.inputs):
@@ -959,7 +990,9 @@ only implemented for SISO functions.")
         return np.squeeze(gain)
 
 # c2d function contributed by Benjamin White, Oct 2012
-def _c2dmatched(sysC, Ts):
+
+
+def _c2d_matched(sysC, Ts):
     # Pole-zero match method of continuous to discrete time conversion
     szeros, spoles, sgain = tf2zpk(sysC.num[0][0], sysC.den[0][0])
     zzeros = [0] * len(szeros)
@@ -967,35 +1000,37 @@ def _c2dmatched(sysC, Ts):
     pregainnum = [0] * len(szeros)
     pregainden = [0] * len(spoles)
     for idx, s in enumerate(szeros):
-        sTs = s*Ts
+        sTs = s * Ts
         z = exp(sTs)
         zzeros[idx] = z
-        pregainnum[idx] = 1-z
+        pregainnum[idx] = 1 - z
     for idx, s in enumerate(spoles):
-        sTs = s*Ts
+        sTs = s * Ts
         z = exp(sTs)
         zpoles[idx] = z
-        pregainden[idx] = 1-z
-    zgain = np.multiply.reduce(pregainnum)/np.multiply.reduce(pregainden)
-    gain = sgain/zgain
+        pregainden[idx] = 1 - z
+    zgain = np.multiply.reduce(pregainnum) / np.multiply.reduce(pregainden)
+    gain = sgain / zgain
     sysDnum, sysDden = zpk2tf(zzeros, zpoles, gain)
     return TransferFunction(sysDnum, sysDden, Ts)
 
 # Utility function to convert a transfer function polynomial to a string
 # Borrowed from poly1d library
-def _tfpolyToString(coeffs, var='s'):
+
+
+def _tf_polynomial_to_string(coeffs, var='s'):
     """Convert a transfer function polynomial to a string"""
 
     thestr = "0"
 
     # Compute the number of coefficients
-    N = len(coeffs)-1
+    N = len(coeffs) - 1
 
     for k in range(len(coeffs)):
         coefstr = '%.4g' % abs(coeffs[k])
         if coefstr[-4:] == '0000':
             coefstr = coefstr[:-5]
-        power = (N-k)
+        power = (N - k)
         if power == 0:
             if coefstr != '0':
                 newstr = '%s' % (coefstr,)
@@ -1032,7 +1067,7 @@ def _tfpolyToString(coeffs, var='s'):
     return thestr
 
 
-def _addSISO(num1, den1, num2, den2):
+def _add_siso(num1, den1, num2, den2):
     """Return num/den = num1/den1 + num2/den2.
 
     Each numerator and denominator is a list of polynomial coefficients.
@@ -1045,7 +1080,7 @@ def _addSISO(num1, den1, num2, den2):
     return num, den
 
 
-def _convertToTransferFunction(sys, **kw):
+def _convert_to_transfer_function(sys, **kw):
     """Convert a system to transfer function form (if needed).
 
     If sys is already a transfer function, then it is returned.  If sys is a
@@ -1053,8 +1088,8 @@ def _convertToTransferFunction(sys, **kw):
     returned.  If sys is a scalar, then the number of inputs and outputs can be
     specified manually, as in:
 
-    >>> sys = _convertToTransferFunction(3.) # Assumes inputs = outputs = 1
-    >>> sys = _convertToTransferFunction(1., inputs=3, outputs=2)
+    >>> sys = _convert_to_transfer_function(3.) # Assumes inputs = outputs = 1
+    >>> sys = _convert_to_transfer_function(1., inputs=3, outputs=2)
 
     In the latter example, sys's matrix transfer function is [[1., 1., 1.]
                                                               [1., 1., 1.]].
@@ -1062,7 +1097,7 @@ def _convertToTransferFunction(sys, **kw):
     If sys is an array-like type, then it is converted to a constant-gain
     transfer function.
 
-    >>> sys = _convertToTransferFunction([[1., 0.], [2., 3.]])
+    >>> sys = _convert_to_transfer_function([[1., 0.], [2., 3.]])
 
     In this example, the numerator matrix will be
        [[[1.0], [0.0]], [[2.0], [3.0]]]
@@ -1078,12 +1113,12 @@ def _convertToTransferFunction(sys, **kw):
 
         return sys
     elif isinstance(sys, StateSpace):
-        
-        if 0==sys.states:
+
+        if 0 == sys.states:
             # Slycot doesn't like static SS->TF conversion, so handle
             # it first.  Can't join this with the no-Slycot branch,
             # since that doesn't handle general MIMO systems
-            num = [[[sys.D[i,j]] for j in range(sys.inputs)] for i in range(sys.outputs)]
+            num = [[[sys.D[i, j]] for j in range(sys.inputs)] for i in range(sys.outputs)]
             den = [[[1.] for j in range(sys.inputs)] for i in range(sys.outputs)]
         else:
             try:
@@ -1111,14 +1146,14 @@ def _convertToTransferFunction(sys, **kw):
 
             except ImportError:
                 # If slycot is not available, use signal.lti (SISO only)
-                if (sys.inputs != 1 or sys.outputs != 1):
-                    raise TypeError("No support for MIMO without slycot")
+                if sys.inputs != 1 or sys.outputs != 1:
+                    raise TypeError("No support for MIMO without slycot.")
 
                 # Do the conversion using sp.signal.ss2tf
                 # Note that this returns a 2D array for the numerator
                 num, den = sp.signal.ss2tf(sys.A, sys.B, sys.C, sys.D)
-                num = squeeze(num) # Convert to 1D array
-                den = squeeze(den) # Probably not needed
+                num = squeeze(num)  # Convert to 1D array
+                den = squeeze(den)  # Probably not needed
 
         return TransferFunction(num, den, sys.dt)
 
@@ -1230,7 +1265,7 @@ def tf(*args):
     """
 
     if len(args) == 2 or len(args) == 3:
-       return TransferFunction(*args)
+        return TransferFunction(*args)
     elif len(args) == 1:
         from .statesp import StateSpace
         sys = args[0]
@@ -1239,11 +1274,12 @@ def tf(*args):
         elif isinstance(sys, TransferFunction):
             return deepcopy(sys)
         else:
-            raise TypeError("tf(sys): sys must be a StateSpace or \
-TransferFunction object.  It is %s." % type(sys))
+            raise TypeError("tf(sys): sys must be a StateSpace or TransferFunction object. "
+                            "It is %s." % type(sys))
     else:
         raise ValueError("Needs 1 or 2 arguments; received %i." % len(args))
 
+
 def ss2tf(*args):
     """ss2tf(sys)
 
@@ -1309,20 +1345,20 @@ def ss2tf(*args):
     from .statesp import StateSpace
     if len(args) == 4 or len(args) == 5:
         # Assume we were given the A, B, C, D matrix and (optional) dt
-        return _convertToTransferFunction(StateSpace(*args))
+        return _convert_to_transfer_function(StateSpace(*args))
 
     elif len(args) == 1:
         sys = args[0]
         if isinstance(sys, StateSpace):
-            return _convertToTransferFunction(sys)
+            return _convert_to_transfer_function(sys)
         else:
-            raise TypeError("ss2tf(sys): sys must be a StateSpace object.  It \
-is %s." % type(sys))
+            raise TypeError("ss2tf(sys): sys must be a StateSpace object.  It is %s." % type(sys))
     else:
         raise ValueError("Needs 1 or 4 arguments; received %i." % len(args))
 
+
 def tfdata(sys):
-    '''
+    """
     Return transfer function data objects for a system
 
     Parameters
@@ -1334,43 +1370,44 @@ def tfdata(sys):
     -------
     (num, den): numerator and denominator arrays
         Transfer function coefficients (SISO only)
-    '''
-    tf = _convertToTransferFunction(sys)
+    """
+    tf = _convert_to_transfer_function(sys)
 
-    return (tf.num, tf.den)
+    return tf.num, tf.den
 
-def _cleanPart(data):
-    '''
-    Return a valid, cleaned up numerator or denominator 
+
+def _clean_part(data):
+    """
+    Return a valid, cleaned up numerator or denominator
     for the TransferFunction class.
-    
+
     Parameters
     ----------
     data: numerator or denominator of a transfer function.
-    
+
     Returns
     -------
     data: list of lists of ndarrays, with int converted to float
-    '''
+    """
     valid_types = (int, float, complex, np.number)
     valid_collection = (list, tuple, ndarray)
 
     if (isinstance(data, valid_types) or
-        (isinstance(data, ndarray) and data.ndim == 0)):
+            (isinstance(data, ndarray) and data.ndim == 0)):
         # Data is a scalar (including 0d ndarray)
         data = [[array([data])]]
     elif (isinstance(data, ndarray) and data.ndim == 3 and
-          isinstance(data[0,0,0], valid_types)):
-        data = [ [ array(data[i,j]) 
-            for j in range(data.shape[1])]
-            for i in range(data.shape[0])]
+          isinstance(data[0, 0, 0], valid_types)):
+        data = [[array(data[i, j])
+                 for j in range(data.shape[1])]
+                for i in range(data.shape[0])]
     elif (isinstance(data, valid_collection) and
             all([isinstance(d, valid_types) for d in data])):
         data = [[array(data)]]
     elif (isinstance(data, (list, tuple)) and
           isinstance(data[0], (list, tuple)) and
-              (isinstance(data[0][0], valid_collection) and 
-               all([isinstance(d, valid_types) for d in data[0][0]]))):
+          (isinstance(data[0][0], valid_collection) and
+           all([isinstance(d, valid_types) for d in data[0][0]]))):
         data = list(data)
         for j in range(len(data)):
             data[j] = list(data[j])
@@ -1379,16 +1416,14 @@ def _cleanPart(data):
     else:
         # If the user passed in anything else, then it's unclear what
         # the meaning is.
-        raise TypeError("The numerator and denominator inputs must be \
-scalars or vectors (for\nSISO), or lists of lists of vectors (for SISO or \
-MIMO).")
+        raise TypeError("The numerator and denominator inputs must be scalars or vectors "
+                        "(for\nSISO), or lists of lists of vectors (for SISO or MIMO).")
 
     # Check for coefficients that are ints and convert to floats
     for i in range(len(data)):
         for j in range(len(data[i])):
             for k in range(len(data[i][j])):
-                if (isinstance(data[i][j][k], (int, np.int))):
+                if isinstance(data[i][j][k], (int, np.int)):
                     data[i][j][k] = float(data[i][j][k])
-                
+
     return data
-    
