@@ -17,13 +17,17 @@ sys = ss(A, B, C, 0);
 
 # Step response for the system
 figure(1)
-T, yout = step(sys)
-plot(T, yout)
+yout, T = step(sys)
+plot(T.T, yout.T)
 
 # Bode plot for the system
 figure(2)
-bode(sys, logspace(-2, 2))
+mag,phase,om = bode(sys, logspace(-2, 2),Plot=True)
 
 # Nyquist plot for the system
 figure(3)
 nyquist(sys, logspace(-2, 2))
+
+# Root lcous plut for the system
+figure(4)
+rlocus(sys)
